
const bool colortex4MipmapEnabled = true;
const bool colortex6MipmapEnabled = true;
const float bloom_spread = 16.0;
const float bloom_intensity = 1.0;
varying vec2 texcoord;

vec4 apply_bloom(sampler2D tex, vec2 texcoord, vec2 blurDirection, float lod)
{
#ifdef STAGE0
    // Compute texel size in full resolution space.
    vec2 texelSize = bloom_spread / viewResolution;
    // Compensate for the half-texel shift: subtract half a texel in the x-direction.
    vec2 adjustedTexcoord = texcoord + vec2(0.5 * texelSize.x, 0.0);
    // Use adjustedTexcoord in the sampling loop.
    float weights[9] = float[](0.05, 0.09, 0.12, 0.15, 0.18, 0.15, 0.12, 0.09, 0.05);
    vec4 sum = vec4(0.0);
    for (int i = -4; i <= 4; ++i)
    {
        vec2 offset = blurDirection * float(i) * texelSize;
        sum += textureLod(tex, adjustedTexcoord + offset, lod) * weights[i + 4];
    }
    return sum * 0.25;
#else
    // For the quarter-resolution pass, a similar compensation might be needed if you see an offset.
    vec2 texelSize = bloom_spread / viewResolution;
    vec2 adjustedTexcoord = texcoord + vec2(0.5 * texelSize.x, 0.0); // Adjust if necessary.
    float weights[7] = float[](0.05, 0.09, 0.12, 0.15, 0.12, 0.09, 0.05);
    vec4 sum = vec4(0.0);
    for (int i = -3; i <= 3; ++i)
    {
        vec2 offset = blurDirection * float(i) * texelSize;
        sum += textureLod(tex, adjustedTexcoord + offset, lod) * weights[i + 3];
    }
    return sum * 0.333333;
#endif
}

/* DRAWBUFFERS:6 */

void main()
{
    vec3 result;
#ifdef STAGE0
    // Horizontal pass on the full-resolution texture.
    result = apply_bloom(colortex4, texcoord, vec2(1.0, 0.0), 4.0).rgb;
#else
    // Vertical pass on the quarter-resolution texture using the optimized approach.
    result = apply_bloom(colortex6, texcoord, vec2(0.0, 1.0), 2.0).rgb;
#endif

    gl_FragData[0].rgb = result;
}
