

#define SIMPLEFOG
#define WATERSTAGE

varying mat3 tbnMatrix;
varying vec4 color;
varying float iswater;
varying vec4 lmtexcoord;
varying vec3 torch;
varying vec3 viewVector;
varying vec3 normal0;
varying vec3 normal1;
flat varying vec4 labvalue;

#ifdef PHYSICS_MOD
    #include "/lib/physicsmod.glsl"

#endif

#include "/lib/sky_function.glsl"
#include "/lib/light.glsl"
#ifdef WATER_SHADOWS

const vec2 shadowOffsets[6] = vec2[6](vec2(0.5303, 0.5303), vec2(-0.6250, -0.0000), vec2(0.3536, -0.3536),
                                      vec2(-0.0000, 0.3750), vec2(-0.1768, -0.1768), vec2(0.1250, 0.0000));

vec2 tapLocation(int sampleNumber, int nb, float nbRot, float jitter, float distort)
{
    float alpha = (sampleNumber + jitter) / nb;
    float angle = jitter * TWO_PI + alpha * nbRot * TWO_PI;

    float sin_v, cos_v;

    sin_v = sin(angle);
    cos_v = cos(angle);

    return vec2(cos_v, sin_v) * sqrt(alpha);
}
#endif
#define viewMAD(m, v) (mat3(m) * (v) + (m)[3].xyz)

#define diagonal3(m) vec3((m)[0].x, (m)[1].y, m[2].z)
#define projMAD(m, v) (diagonal3(m) * (v) + (m)[3].xyz)

#include "/lib/water/waterfuncs2.glsl"

#ifdef WATER_PBR
    #if SSR_QUALITY == 0
        #include "/lib/water/ssr_normal.glsl"
    #elif SSR_QUALITY == 1
        #include "/lib/water/ssr_extra_low.glsl"

    #elif SSR_QUALITY == 2
        #include "/lib/water/ssr_low.glsl"
    #else
        #include "/lib/water/ssr_hq.glsl"

    #endif
#else
    #include "/lib/water/ssr_normal.glsl"
#endif

float GetBorderFogMixFactorCylinder(in vec3 eyePlayerPos, in float far, bool isWater)
{

    float mixval = 0.75;

    if (isWater)
        mixval = 0.33;
    float eyeDist = length(eyePlayerPos.xz);
    float borderFogFactor = smoothstep(far * mixval, far, eyeDist);

    return borderFogFactor;
}

vec4 getSpecularProperties(vec2 tcoord)
{
    float reflectance = labvalue.x;
    float smoothness = labvalue.y;
    float sss = labvalue.z;
    float emissives = 0;
#ifdef LAB_PBR
    if (iswater < 0.99)
    {

        // vec4 specularTex = texture2DLod(specular, tcoord, 0);
        vec4 specularTex = texture(specular, tcoord);

        smoothness = specularTex.x;
        sss = specularTex.z;
        reflectance = specularTex.y;
        emissives = specularTex.w;
    }
#endif
    return vec4(sss, emissives, reflectance, smoothness);
}

vec2 computeRefractedCoord(vec3 fragPos0, vec3 fragPos1, vec2 texcoord, vec3 normalFlat, vec3 normal, float F0)
{
    vec3 I = normalize(fragPos0);
    float eta = 1.0 / (1.0 + sqrt(F0)) / (1.0 - sqrt(F0));
    float NoI = dot(normal, I);
    float eta2 = eta * eta;
    float NoI2 = NoI * NoI;
    float k = 1.0 - eta2 * (1.0 - NoI2);

    if (k < 0.0)
    {
        return vec2(0.0);
    }

    float sqrtK = sqrt(k);
    vec3 normFlatDiff = (normalFlat - normal);
#ifdef PHYSICS_MOD
    normFlatDiff *= 8.0;
#endif
    vec3 refractedDir = eta * (dot(normalFlat, I) * normalFlat - NoI * normal) + sqrtK * normFlatDiff + eta * I;

    vec3 refractedPos = fragPos0 + normalize(refractedDir) * distance(fragPos0, fragPos1);
    vec3 screenPos = toClipSpace3(refractedPos);

    return screenPos.xy - texcoord;
}
vec2 computeRefractedCoord2(vec3 fragPos0, vec3 fragPos1, vec2 texcoord, vec2 texcoord2, vec3 normalFlat, vec3 normal, float F0)
{
    vec3 I = normalize(fragPos0);
    float eta = 1.0 / (1.0 + sqrt(F0)) / (1.0 - sqrt(F0));
    float NoI = dot(normal, I);
    float eta2 = eta * eta;
    float NoI2 = NoI * NoI;
    float k = 1.0 - eta2 * (1.0 - NoI2);

    if (k < 0.0)
    {
        return texcoord;
    }

    float sqrtK = sqrt(k);
    vec3 normFlatDiff = normalFlat - normal;

    vec3 refractedDir = eta * (dot(normalFlat, I) * normalFlat - NoI * normal) + sqrtK * normFlatDiff + eta * I;

    vec3 refractedPos = fragPos0 + normalize(refractedDir) * distance(fragPos0, fragPos1);
    vec3 screenPos = toClipSpace3(refractedPos);

    return texcoord + (screenPos.xy - texcoord2);
}

vec3 Reproject(vec3 position)
{
    bool isHand = position.z < 0.6;
    if (position.z < 1.0 && !isHand)
    {
        position = ViewSpaceFromScreenSpace(position, gbufferProjectionInverse);
        position = mat3(gbufferModelViewInverse) * position + gbufferModelViewInverse[3].xyz;
        position += (cameraPosition - previousCameraPosition);
        position = mat3(gbufferPreviousModelView) * position + gbufferPreviousModelView[3].xyz;
        position = ScreenSpaceFromViewSpace(position, gbufferPreviousProjection);
    }
    else
    {
        position = ViewSpaceFromScreenSpace(position, gbufferProjectionInverse);
        position = mat3(gbufferModelViewInverse) * position + gbufferModelViewInverse[3].xyz;
        position = mat3(gbufferPreviousModelView) * position + gbufferPreviousModelView[3].xyz;
        position = ScreenSpaceFromViewSpace(position, gbufferPreviousProjection);
    }

    return position;
}
////
float phaseg(float x, float g)
{
    float gg = g * g;
    return (gg * -0.25 + 0.25) * pow(-2.0 * (g * x) + (gg + 1.0), -1.5) / PI;
}
float phaseRayleigh(float cosTheta)
{
    return cosTheta * 0.18 + 0.51;
}

void waterNoLoop(float rayLength, float dY, float estEyeDepth, vec3 refractedSunVec, vec3 ambient, vec3 sunColor, inout vec3 absorbance_w, inout vec3 vL_w, vec3 dVWorld)
{
    const vec3 totEpsilon = vec3(0.69, 0.15, 0.18);

    const vec3 dirtScatterCoef = 0.1 * vec3(0.2, 0.6, 0.8);

    vec3 water_coefficientRayleigh = vec3(1.0 * 1e-6, 2.5 * 1e-6, 3.5 * 1e-6);

    float SdotV = dot(sunVec, normalize(dVWorld));
    vec3 rL_Water = water_coefficientRayleigh;
    vec3 mieScatterFactor_Water = dirtScatterCoef;
    vec3 ScatterCoeff_water = totEpsilon + mieScatterFactor_Water;

    float mie_water = phaseg(SdotV, 0.4);
    float Rayleigh = phaseRayleigh(SdotV);
    float sunCaustics = 0.25; // Compensates for the caustics we removed
    float sh = 1.0;           // Compensates for the shadows we removed

    // Average depth factor along the ray
    float avg_d = 0.5;
    float attenuationDepth = rayLength;

    vec3 sun = sunColor * sh * sunCaustics * PI;

    vec3 ambientMul_water = exp(-max(estEyeDepth - dY * avg_d, 0.0) * rL_Water);
    vec3 sunMul_water = exp(-max(estEyeDepth - dY * avg_d, 0.0) / abs(refractedSunVec.y) * totEpsilon);

    vec3 mie_water_mult = mie_water * sunMul_water;
    vec3 rayleighContribution_Water = Rayleigh * rL_Water;
    vec3 mieScatteringContribution_Water = mieScatterFactor_Water * mie_water_mult;
    vec3 totalScatter_Water = rayleighContribution_Water + mieScatteringContribution_Water;

    vec3 sunContribution_Water = sun * totalScatter_Water;
    vec3 ambientContribution_Water = ambient * ambientMul_water * (rL_Water + mieScatterFactor_Water);
    vec3 light_water = sunContribution_Water + ambientContribution_Water;

    // Calculate absorbance over the entire ray length
    vec3 absorbance = exp(-ScatterCoeff_water * attenuationDepth);

    // Update volumetric light accumulation
    vL_w += (light_water - light_water * absorbance) / (ScatterCoeff_water + 1e-8) * absorbance_w;
    absorbance_w *= absorbance;
}

/*RENDERTARGETS: 1,5,13*/
void main()
{

#ifdef PHYSICS_MOD

    physics_waveData = physics_wavePixel(physics_localPosition.xz, physics_localWaviness, physics_iterationsNormal, physics_gameTime);
    vec3 physics_normal = normalize(gl_NormalMatrix * physics_waveData.normal);
#endif
    bool waterMask = (iswater > 0.99);

    gl_FragData[1] = waterMask ? vec4(1, 0, 0, 0.95) : vec4(0, 0, 1, 0.1);

    /////
    const vec3 wcolor = vec3(0.18, 0.35, 0.43) * 1.75;
    vec2 texc = gl_FragCoord.xy * texelSize;
    vec2 texcoord = lmtexcoord.xy;
    vec3 prevtexc = Reproject(vec3(texc, gl_FragCoord.z));
    float mask = texture(colortex4, prevtexc.xy).a;
    bool isGlassWater = mask > 0.9;
    vec4 materialProps = getSpecularProperties(texcoord);
    float f0 = materialProps.z;
    vec4 outcolor;

    /////

    vec3 normFragpos = normalize(viewVector);
    vec3 normal = normal1;
    vec3 normalWorld = normal0;
    vec3 normalTex = vec3(0.0);

#ifdef NORMAL_MAPS
    normalTex = texture(normals, texcoord).rgb;
    normal = normalize(tbnMatrix * (normalTex * 2.0 - 1.0));
    normalWorld = worldToView(normal);
#endif
    vec3 p3 = (mat3(gbufferModelViewInverse) * viewVector) + gbufferModelViewInverse[3].xyz;

    if (waterMask)
    {

        vec3 posxz = p3 + cameraPosition;

        normalTex = waterNormal(posxz.xz - posxz.y, normFragpos);
        normal = normalize(tbnMatrix * (normalTex * 2.0 - 1.0));
        normalWorld = worldToView(normal);
    }
#ifdef PHYSICS_MOD

    normal = mix(viewToWorld(physics_normal), normal, 0.1);
    normalWorld = mix(physics_normal, normalWorld, 0.1);
#endif

#ifdef REFRACTION
    vec2 refractionMain = computeRefractedCoord(viewVector, toScreenSpace(vec3(texc, texture(depthtex1, texc).x)), texc, normal0, normalWorld, f0);

#endif

    vec4 albedo = texture2D(texture, texcoord);
    albedo.rgb = toLinear(albedo.rgb * color.rgb);
#ifndef VANILLA_WATER

    outcolor = waterMask ? vec4(toLinear(mix(wcolor, color.rgb, 0.25)), 0.5) : albedo;
#else
    outcolor = waterMask ? vec4(albedo.rgb, 0.5) : albedo;
#endif
    float alpha0 = waterMask ? 0.1 : outcolor.a;
#ifdef REFRACTION
    gl_FragData[2] = vec4(refractionMain, 0.0, outcolor.a); // Normals
#endif

    // float diffuseSun = mix(max(dot(normal, lightPos), 0.002), max(dot(normal, lightPos), 0.0), outcolor.a) * lmtexcoord.w;
    float diffuseSun = max(dot(normal, lightPos), 0.0);
    diffuseSun = mix(max(diffuseSun, 1.0 - outcolor.a), diffuseSun, outcolor.a);
    diffuseSun = diffuseSun * lmtexcoord.w;

    vec3 direct = sunLight;
    float shading = 1.0;

#ifdef OVERWORLD
    #ifdef WATER_SHADOWS
    if (diffuseSun > 0.0)
    {
        float noise = R2_dither(gl_FragCoord.xy);

        vec3 ProjShadowPos = mat3(shadowModelView) * p3 + shadowModelView[3].xyz;
        ProjShadowPos = diagonal3(shadowProjection) * ProjShadowPos + shadowProjection[3].xyz;

        float distortFactor = calcDistort(ProjShadowPos.xy);
        ProjShadowPos.xy *= distortFactor;

        if (all(lessThan(abs(ProjShadowPos.xy), (vec2(1.0, 1.0) - 1.5 / shadowMapResolution))) &&
            abs(ProjShadowPos.z) < 6)
        {
            float threshMulMult = mix(2048, 4096, float(eyeAltitude > 400));
            float threshMulMult2 = mix(threshMulMult, 8192, float(eyeAltitude > 500));
            float threshMul = max(threshMulMult2 / shadowMapResolution * shadowDistance / 100.0, 0.95);
            float distortThresh = clamp(sqrt(1.0 - diffuseSun * diffuseSun) / diffuseSun, 0.0, 0.25) / distortFactor;
            float diffthresh = distortThresh / 6000.0 * threshMul;

            ProjShadowPos = ProjShadowPos * vec3(0.5, 0.5, 0.5 / 6.0) + 0.5;

            shading = 0.0;
            float rdMul = 4.0 / shadowMapResolution;
            for (int i = 0; i < 9; i++)
            {
                vec2 offsetS = tapLocation(i, 9, 2.0, noise, 0.0);

                float weight = 1.0 + (i + noise) * rdMul / 9.0 * shadowMapResolution;
                vec3 shadowCoordWeighted = vec3(ProjShadowPos + vec3(rdMul * offsetS, -diffthresh * weight));
                float shadow0 = texture(shadowtex1, shadowCoordWeighted);
        #ifdef COLORED_SHADOWS
                float shadow1 = texture(shadow, shadowCoordWeighted);

                float transparentshadow = distance(shadow1, shadow0);
                vec4 shadowcolor = texture(shadowcolor0, shadowCoordWeighted.xy);

                //  shading += shadow1;

                if (transparentshadow > 0)
                {
                    shading += clamp(shadow1 + clamp((1 - shadowcolor.a), 0.0, 1.0), 0.0, 1.0) / 9;
                }
                else
                {
                    shading += shadow1 / 9;
                }
        #else
                shading += shadow0 / 9;

        #endif
            }
        }

        shading = mix(0.0, shading, shadowblend);

        diffuseSun = min(shading, diffuseSun) * lmtexcoord.w;

        direct *= shading;
    }
    #endif
#endif

    direct *= (1 - rainStrength * 0.9);
    diffuseSun *= (1 - rainStrength * 0.9);
    vec3 ambient = calcAmbientLight(normal, lmtexcoord.zw, atmosphereUp, diffuseSun, torch) * color.a;
    outcolor.rgb += (materialProps.y * 8.0 * outcolor.rgb * outcolor.rgb);

    float roughness = pow(1.0 - materialProps.w, 2.0);
    if (waterMask)
    {
        roughness = 0.01;
        f0 = 0.002;
    }

    /////////

    vec3 diffuse = ambient * outcolor.rgb * diffuseSun;

    // Translucency effect
    vec3 transmission = ambient * outcolor.rgb * (1.0 - outcolor.a);

    // Combine diffuse reflection and transmission
    outcolor.rgb = mix(transmission, diffuse, outcolor.a);

#if SSR_QUALITY > 0

    vec3 f03 = (int(materialProps.z * 255.0) > 229) ? albedo.rgb : vec3(materialProps.z);

    #ifdef HARDCODED_METALS

    if (distance(f03, albedo.rgb) <= 0.0)
    {
        outcolor.rgb = ambient;
    }

    #endif
#endif

    outcolor = reflections(lmtexcoord.w, normal, viewVector, materialProps, outcolor, diffuseSun, albedo.rgb, alpha0);

    if (!waterMask && isGlassWater || (isEyeInWater == 1 && !waterMask))
    {
        float estEyeDark = clamp((eyeBrightnessSmooth.y / 255.0), 0.0, 1.0);

        float estEyeDepth = clamp((14.0 - eyeBrightnessSmooth.y / 255.0 * 16.0) / 14.0, 0.0, 1.0);
        estEyeDepth *= estEyeDepth * estEyeDepth * 32.0;

        vec3 fragpos = viewVector;

        float rayLength = length(fragpos);
        if (isEyeInWater == 1)
        {
            float maxZ = (min(rayLength, 32.0) / (1e-8 + rayLength));

            rayLength = rayLength * maxZ;
        }

        float dY = normalize(p3).y * rayLength;
        vec3 refractedSunVec = refract(lightPos, -vec3(0.0, 1.0, 0.0), 0.9);
        vec3 ambient1 = atmosphereUp;
        if (isEyeInWater == 0)
            ambient1 *= 0.5;
        vec3 sunColor = sunLight;
        vec3 absorbance_w1 = vec3(1.0);
        vec3 vL_w1 = vec3(0.0);
        // outcolor.a = 1.0;
        waterNoLoop(rayLength, dY, estEyeDepth, refractedSunVec, ambient1, sunColor, absorbance_w1, vL_w1, p3);
        if (isEyeInWater == 0)
            vL_w1 *= estEyeDark;
        outcolor.rgb *= absorbance_w1;
        outcolor.rgb += vL_w1;
    }
    if (waterMask && !(normal1.g < -0.1) && isEyeInWater == 1)
    {
        gl_FragData[1] = vec4(0, 0, 0, 0.0);
        outcolor.a = 0;
        gl_FragData[2].a = 0;
    }

#if DEBUG_MODE == 1
    outcolor = vec4(ambient, 1.0);
#elif DEBUG_MODE == 2
    outcolor = vec4(reflected, 1.0);
#elif DEBUG_MODE == 3
    outcolor = vec4(normal, 1.0);
#elif DEBUG_MODE == 4
    outcolor = vec4(vec3(diffuseSun), 1.0);
#elif DEBUG_MODE == 5
    outcolor = vec4(torch, 1.0);
#endif

#ifdef DISTANT_HORIZONS

    float mixfactor = GetBorderFogMixFactorCylinder(p3, far, iswater >= 0.9);
    outcolor.a = mix(outcolor.a, 0, mixfactor);
#endif

#ifdef PHYSICS_MOD
    outcolor = mix(outcolor, vec4(physics_foamColor * diffuseSun, 1.0), physics_waveData.foam);
#endif
    //  outcolor.a = 1;

    vec4 sky = skyFromTexLod(p3, colortex6, 0);
    sky.rgb += sky.a * sunLight;
#ifdef BETTER_TRANSLUCENT_FOG
    applySkyBlend2(outcolor, sky.rgb, p3, far);
#endif
    // outcolor = vec4(normal, 1);
    gl_FragData[0] = clamp(outcolor, 0, 1000);
}
