

flat varying vec3 ambientUp;
flat varying vec3 ambientDown;
flat varying vec3 ambientLeft;
flat varying vec3 ambientRight;
flat varying vec3 ambientB;
flat varying vec3 ambientF;
flat varying float ambientA;
flat varying vec4 sunColor;
varying vec2 texcoord;

#include "/lib/sky_function.glsl"

vec3 GenerateUnitVector(vec2 xy)
{
    const float TAU = 6.28318530718; // 2*pi
    xy.x *= TAU;
    xy.y = xy.y * 2.0 - 1.0;
    float r = sqrt(1.0 - xy.y * xy.y);
    return vec3(vec2(sin(xy.x), cos(xy.x)) * r, xy.y);
}

void calculateLighting2(
    out vec3 ambientUp,
    out vec3 ambientLeft,
    out vec3 ambientRight,
    out vec3 ambientB,
    out vec3 ambientF,
    out vec3 ambientDown,
    out float ambientA,
    out vec4 sunColor)
{
    // Precompute sun visibility.
    float clampedSunElevation = clamp(sunElevation, 0.0, 0.05);
    float t = clampedSunElevation / 0.05;
    float sunVis = t * t;
    sunColor = vec4(sunLight, sunVis);

    // Sample grid setup.
    const ivec2 samples = ivec2(32, 16);
    const vec2 invSamples = vec2(1.0 / float(samples.x), 1.0 / float(samples.y));
    const float sampleWeight = 2.0 / float(samples.x * samples.y);

    // Initialize ambient accumulators.
    ambientRight = ambientUp = ambientF = vec3(0.0);
    ambientLeft = ambientDown = ambientB = vec3(0.0);

    // Loop over the sample grid.
    for (int x = 0; x < samples.x; ++x)
    {
        for (int y = 0; y < samples.y; ++y)
        {
            vec2 uv = (vec2(x, y) + 0.5) * invSamples;
            vec3 dir = GenerateUnitVector(uv);
            dir.y = abs(dir.y); // enforce upward direction

            vec3 skySample = skyFromTex2(dir, colortex6).rgb;

            // Precompute directional weights.
            float posX = max(dir.x, 0.0);
            float posY = max(dir.y, 0.0);
            float posZ = max(dir.z, 0.0);
            float negX = max(-dir.x, 0.0);
            float negZ = max(-dir.z, 0.0);

            ambientRight += skySample * posX;
            ambientUp += skySample * posY;
            ambientF += skySample * posZ;
            ambientLeft += skySample * negX;
            ambientB += skySample * negZ;
        }
    }

    // Apply sample weight.
    ambientRight *= sampleWeight;
    ambientUp *= sampleWeight;
    ambientF *= sampleWeight;
    ambientLeft *= sampleWeight;
    ambientB *= sampleWeight;

    // Fake skylight bounce.
    const float fakeBounceAlbedo = 0.5;
    ambientRight += ambientUp * fakeBounceAlbedo * 0.5;
    ambientF += ambientUp * fakeBounceAlbedo * 0.5;
    ambientLeft += ambientUp * fakeBounceAlbedo * 0.5;
    ambientDown += ambientUp * fakeBounceAlbedo;
    ambientB += ambientUp * fakeBounceAlbedo * 0.5;

    // Compute light direction.
    float lightDir = (sunVis >= 1e-5) ? 1.0 : -1.0;

    vec3 sunLightLocal = sunLight;

    vec3 bouncedSun = sunLightLocal * 0.001;

    // Precompute bounce clamping values.
    float clampUp = clamp(-lightDir * sunVec.y + 3.0, 0.0, 4.0);
    float clampLeft = clamp(lightDir * sunVec.x + 3.0, 0.0, 4.0);
    float clampRight = clamp(-lightDir * sunVec.x + 3.0, 0.0, 4.0);
    float clampBack = clamp(-lightDir * sunVec.z + 3.0, 0.0, 4.0);
    float clampFront = clamp(lightDir * sunVec.z + 3.0, 0.0, 4.0);
    float clampDown = clamp(lightDir * sunVec.y + 3.0, 0.0, 4.0);

    ambientUp += bouncedSun * clampUp;
    ambientLeft += bouncedSun * clampLeft;
    ambientRight += bouncedSun * clampRight;
    ambientB += bouncedSun * clampBack;
    ambientF += bouncedSun * clampFront;
    ambientDown += bouncedSun * clampDown * 0.7;

    // Compute ambient overall luminance.
    ambientA = luma((ambientUp + ambientLeft + ambientRight + ambientB + ambientF + ambientDown) / 6.0);
}

void main()
{

    texcoord = gl_MultiTexCoord0.xy;

    calculateLighting2(ambientUp, ambientLeft, ambientRight, ambientB, ambientF, ambientDown, ambientA, sunColor);

    gl_Position = ftransform();
}
