#extension GL_ARB_gpu_shader5 : enable

varying vec2 texcoord;

vec2 HashVec(vec2 d)
{
    const vec2 k1 = vec2(123.4, 234.5);
    const vec2 k2 = vec2(234.5, 345.6);
    const float MULT = 58322.51;
    const float TIME_MULT = 2.0;

    vec2 s = sin(vec2(dot(d, k1), dot(d, k2)));
    return sin(s * MULT + frameTimeCounter * TIME_MULT);
}

float Octave(vec2 uv, float gridSize)
{
    // Precompute the scaled coordinate to avoid redundant multiplication.
    vec2 scaledUV = uv * gridSize;
    vec2 gridUV = fract(scaledUV);
    vec2 cellId = floor(scaledUV);
    vec2 smoothUV = smoothstep(0.0, 1.0, gridUV);

    // Precompute wrapped cell IDs for the four corners.
    vec2 cell00 = mod(cellId, gridSize);
    vec2 cell10 = mod(cellId + vec2(1.0, 0.0), gridSize);
    vec2 cell01 = mod(cellId + vec2(0.0, 1.0), gridSize);
    vec2 cell11 = mod(cellId + vec2(1.0, 1.0), gridSize);

    // Precompute the UV offsets for interpolation.
    vec2 uv10 = gridUV - vec2(1.0, 0.0);
    vec2 uv01 = gridUV - vec2(0.0, 1.0);
    vec2 uv11 = gridUV - vec2(1.0, 1.0);

    // Compute dot products with hashed vectors.
    float n00 = dot(HashVec(cell00), gridUV);
    float n10 = dot(HashVec(cell10), uv10);
    float n01 = dot(HashVec(cell01), uv01);
    float n11 = dot(HashVec(cell11), uv11);

    // Perform bilinear interpolation.
    float nx0 = mix(n00, n10, smoothUV.x);
    float nx1 = mix(n01, n11, smoothUV.x);
    return mix(nx0, nx1, smoothUV.y);
}

float WaterMap(vec2 pos)
{
    float gradient = 0.0;
    float frequency = 8.0;
    float amplitude = 6.0;

    for (int i = 0; i < 6; i++)
    {
        gradient += (Octave(pos, frequency) + 0.5) / amplitude;
        frequency *= 3.0;
        amplitude *= 1.75;
    }

    return gradient * 0.75;
}
float WaterMapFast(vec2 pos)
{
    float gradient = 0.0;
    float frequency = 8.0;
    float amplitude = 6.0;

    // Use fewer iterations for a faster approximation.
    for (int i = 0; i < 4; i++)
    {
        gradient += (Octave(pos, frequency) + 0.5) / amplitude;
        frequency *= 3.0;
        amplitude *= 1.75;
    }

    return gradient * 0.75;
}

float CausticsMap(float h, vec2 pos)
{
    vec2 e = vec2(0.05, 0.0);
    vec2 r = vec2(0.10, 0.0);
    e = mix(e, r, rainStrength);

    // Use the simplified variant for the offsets.
    float hx = WaterMapFast(pos + e.xy);
    float hy = WaterMapFast(pos + e.yx);
    vec2 grad = vec2(hx - h, hy - h) / e.x;

    float l = length(grad);
    float intensity = mix(0.5 / (l * l + 0.05), 0.5 / (l * l + 0.5), rainStrength);
    return clamp(pow(intensity, 0.5) * 0.25, 0.0, 1.0);
}

void main()
{
    /* DRAWBUFFERS:7*/
    float water = WaterMap(texcoord);
    float casutic = CausticsMap(water, texcoord);

    gl_FragData[0].r = water;
    gl_FragData[0].g = casutic;
}
